==> planner-4.5.0-sources.jar: com/xebialabs/deployit/deployment/rules/RegistriesActor.scala <==
package com.xebialabs.deployit.deployment.rules

import java.io.File

import akka.actor._
import com.xebialabs.deployit.deployment.rules.RegistriesActor.Messages.{RuleRegistryGet, RuleRegistryReturn, StepRegistryGet, StepRegistryReturn}
import com.xebialabs.deployit.plugin.api.rules.Scope
import com.xebialabs.platform.sugar.ResourcesSugar
import com.xebialabs.platform.watcher.FileWatchActor.Messages._
import grizzled.slf4j.Logging

import scala.util.Try
import scalax.file.Path
import nl.javadude.scannit.Scannit

private[rules] object RegistriesActor {

  def props(fileWatchActor: ActorRef) = Props(new RegistriesActor(fileWatchActor))

  object Messages {
    case class RuleRegistryGet()
    case class StepRegistryGet()
    case class RuleRegistryReturn(ruleRegistry: RuleRegistry)
    case class StepRegistryReturn(stepRegistry: StepRegistry)
  }
}

private[rules] class RegistriesActor(fileWatchActor: ActorRef) extends Actor with Registries with ResourceScanningSubscription with ActorLogging {

  var rules : RuleRegistry = createRuleRegistryWithFallback()
  var steps : StepRegistry = stepRegistry()

  override def preStart() {
    subscribeForFileChanges()
  }

  override def receive: Receive = {
    case RuleRegistryGet() => sender() ! RuleRegistryReturn(rules)
    case StepRegistryGet() => sender() ! StepRegistryReturn(steps)
    case Created(_) | Modified(_) | Deleted(_) => rules = createRuleRegistryWithFallback()
    case WatchStopped(path) => subscribeForFileChange(path)
    case StartWatching(path) => log.debug(s"Watching on $path")
  }

  override def subscribeForFileChange(path: Path) = fileWatchActor ! Watch(path)
}

private[deployit] trait Registries extends Logging {

  def createRuleRegistryWithFallback() = Try(ruleRegistry()).getOrElse(fallbackRuleRegistry())

  def fallbackRuleRegistry() = {
    val ruleRegistry = new SimpleRuleRegistry()
    ruleRegistry.registerRule(new Rule("TheOnlyFallbackRule", scope = Scope.PRE_PLAN) {
      override def canFire(scopedObject: AnyRef, context: RulePlanningContext) = true
      override def doFire(scopedObject: AnyRef, context: RulePlanningContext): Unit = throw PlannerException(message = "Rules could not be loaded")
    })
    ruleRegistry
  }

  def ruleRegistry() = {
    try {
      val ruleRegistry = new SimpleRuleRegistry()
      JavaBasedRuleBuilder.fill(ruleRegistry)
      XmlReader.loadAllAndParse(rulesXmlFileName)(new XmlRuleParser(ruleRegistry).ruleFromXml)
      XmlReader.loadAllAndParse(rulesXmlFileName)(new XmlRuleParser(ruleRegistry).disableRulesXml)
      ruleRegistry
    } catch {
      case ex: Exception =>
        error(s"Error while creating rule registry", ex)
        throw ex
    }
  }

  def stepRegistry(scannit: Scannit = Scannit.getInstance()) = {
    val stepRegistry = new SimpleStepRegistry()
    StepClassScanner.fill(stepRegistry, scannit)
    stepRegistry
  }
}

private[rules] trait ResourceScanningSubscription extends ResourcesSugar {

  def subscribeForFileChanges() {
    resources(rulesXmlFileName).filter(_.getProtocol == "file").foreach{ resource =>
      subscribeForFileChange(Path(new File(resource.toURI)))
    }
  }

  def subscribeForFileChange(path: Path)
}

==> planner-5.1.7-sources.jar: com/xebialabs/deployit/deployment/rules/RegistriesActor.scala <==
package com.xebialabs.deployit.deployment.rules

import java.io.File

import akka.actor._
import com.xebialabs.deployit.deployment.rules.RegistriesActor.Messages.{RuleRegistryGet, RuleRegistryReturn, StepRegistryGet, StepRegistryReturn}
import com.xebialabs.deployit.plugin.api.rules.Scope
import com.xebialabs.xlplatform.sugar.ResourcesSugar
import com.xebialabs.xlplatform.watcher.FileWatchActor.Messages._
import grizzled.slf4j.Logging

import scala.util.Try
import scalax.file.Path
import nl.javadude.scannit.Scannit

private[rules] object RegistriesActor {

  def props(fileWatchActor: ActorRef) = Props(new RegistriesActor(fileWatchActor))

  object Messages {
    case class RuleRegistryGet()
    case class StepRegistryGet()
    case class RuleRegistryReturn(ruleRegistry: RuleRegistry)
    case class StepRegistryReturn(stepRegistry: StepRegistry)
  }
}

private[rules] class RegistriesActor(fileWatchActor: ActorRef) extends Actor with Registries with ResourceScanningSubscription with ActorLogging {

  var rules : RuleRegistry = createRuleRegistryWithFallback()
  var steps : StepRegistry = stepRegistry()

  override def preStart() {
    subscribeForFileChanges()
  }

  override def receive: Receive = {
    case RuleRegistryGet() => sender() ! RuleRegistryReturn(rules)
    case StepRegistryGet() => sender() ! StepRegistryReturn(steps)
    case Created(_) | Modified(_) | Deleted(_) => rules = createRuleRegistryWithFallback()
    case WatchStopped(path) => subscribeForFileChange(path)
    case StartWatching(path) => log.debug(s"Watching on $path")
  }

  override def subscribeForFileChange(path: Path) = fileWatchActor ! Watch(path)
}

private[deployit] trait Registries extends Logging {

  def createRuleRegistryWithFallback() = Try(ruleRegistry()).getOrElse(fallbackRuleRegistry())

  def fallbackRuleRegistry() = {
    val ruleRegistry = new SimpleRuleRegistry()
    ruleRegistry.registerRule(new Rule("TheOnlyFallbackRule", scope = Scope.PRE_PLAN) {
      override def canFire(scopedObject: AnyRef, context: RulePlanningContext) = true
      override def doFire(scopedObject: AnyRef, context: RulePlanningContext): Unit = throw PlannerException(message = "Rules could not be loaded")
    })
    ruleRegistry
  }

  def ruleRegistry() = {
    try {
      val ruleRegistry = new SimpleRuleRegistry()
      JavaBasedRuleBuilder.fill(ruleRegistry)
      XmlReader.loadAllAndParse(rulesXmlFileName)(new XmlRuleParser(ruleRegistry).ruleFromXml)
      XmlReader.loadAllAndParse(rulesXmlFileName)(new XmlRuleParser(ruleRegistry).disableRulesXml)
      ruleRegistry
    } catch {
      case ex: Exception =>
        error(s"Error while creating rule registry", ex)
        throw ex
    }
  }

  def stepRegistry(scannit: Scannit = Scannit.getInstance()) = {
    val stepRegistry = new SimpleStepRegistry()
    StepClassScanner.fill(stepRegistry, scannit)
    stepRegistry
  }
}

private[rules] trait ResourceScanningSubscription extends ResourcesSugar {

  def subscribeForFileChanges() {
    resources(rulesXmlFileName).filter(_.getProtocol == "file").foreach{ resource =>
      subscribeForFileChange(Path(new File(resource.toURI)))
    }
  }

  def subscribeForFileChange(path: Path)
}
